/*
 * main.c
 *
 *  Created on: 28-05-2013
 *      Author: Piotr Rzeszut
 *  
 * Description: Przykad komunikacji z wykorzystaniem magistrali RS485
 */

#include <avr/io.h>
#include <util/delay.h>

#include "uart.h"

//zakomentuj ponisz linijk, aby otrzyma kod dla ukadu SLAVE
#define MASTER

//definiujemy adres dla ukadu slave - moemy do sieci podpi wiele ukadw slave
//i kademu nada inny adres - moecie z tym poeksperymentowa
#define SLAVE_ADDR 'A'

//definicja pinu sterujcego kierunkiem transmisji
//podpinamy go do pinu RE na naszej pytce
//RxD (PD0) podpinamy do pinu RO
//TxD (PD1) podpinamy do pinu DI
#define DIR_DDR DDRD
#define DIR_PORT PORTD
#define DIR PD2

#define DIR_NAD() DIR_PORT |= (1<<DIR)
#define DIR_ODB() DIR_PORT &= ~(1<<DIR)

//kompilujemy odpowiedni kod w zalenoci od wyboru
#ifdef MASTER
/*****************************************************************
kod dla ukadu master
*****************************************************************/

//LCD podczopny zgodnie z ustawieniami w includowanym pliku
#include "HD44780.h"

char odebrany;

int main(void){
	
	//inicjalizujemy konieczne ukady
	DIR_DDR |= (1<<DIR);

	DIR_NAD();
	_delay_ms(2);

	LCD_Initalize();
	uart_init(9600);
	sei();

	while(1){
		UCSR0A |= (1<<TXC0);//resetujemy flag zakoczenia nadawania

		uart_putc(27);//esc - reset (taki kod przyjem w swojej implementacji za kod resetu transmisji)
		uart_putc(SLAVE_ADDR);//adres ukadu slave
		uart_putc('2');//pierwsza dana
		uart_putc('3');//druga dana

		while(UCSR0B &(1<<UDRIE0));
		while(!(UCSR0A &(1<<TXC0)));//czekamy na zakoczenie nadawania (oprnienie bufora nadawczego
		//oraz zakoczenie nadawania ostatniego bajtu

		DIR_ODB();//dopiero potem przeczamy si na odbir danych
		for(uint8_t i=0;i<200;i++){//czekamy tylko okrelony czas
		//w przeciwnym razie znaczy, ze nikt nie chce nam odpowiedzie na zapytanie
			_delay_ms(1);
			if((odebrany=uart_getc())!=0)break;
		}
		DIR_NAD();//jeli odebralimy dan (1 bajt) to spowrotem przeczamy w tryb nadawania
		LCD_Clear();//i potem kasujemy lcd i
		if(odebrany!=0){//w zalenoci czy kto nam odpowiedzia wywietlamy komunikat
			LCD_WriteData(odebrany);
		}else{
			LCD_WriteText("Timeout");
		}
		//kolejna prba poczenia z ukadem slave za 1 sekund
		_delay_ms(1000);
		
	}
	
	return 0;
}

#else
/*****************************************************************
kod dla ukadu slave
*****************************************************************/

char odebrany;
uint8_t a,b;

int main(void){
	
	//inicjalizacja sprztu
	DIR_DDR |= (1<<DIR);

	DIR_ODB();
	_delay_ms(2);

	uart_init(9600);
	sei();

	while(1){
		if(uart_getc()==27){//jeli odebralimy kod ESC to znaczy, e kto chce nas o co zapyta
			for(uint8_t i=0;i<200;i++){//czekamy tylko pewien okrelony timeout - moe kto si pomyli
				_delay_ms(1);
				if((odebrany=uart_getc())!=0)break;
			}
			if(odebrany==SLAVE_ADDR){//tylko jeli nadaj do nas
				for(uint8_t i=0;i<200;i++){//czekamy tylko pewien okrelony timeout
					_delay_ms(1);
					if((odebrany=uart_getc())!=0)break;
				}
				a=odebrany-'0';
				for(uint8_t i=0;i<200;i++){//czekamy tylko pewien okrelony timeout
					_delay_ms(1);
					if((odebrany=uart_getc())!=0)break;
				}
				b=odebrany-'0';//odczytalimy 2 bajty danych - 2 cyfry
				DIR_NAD();
				_delay_ms(1);

				UCSR0A |= (1<<TXC0);//resetujemy flag zakoczenia nadawania

				uart_putc(a*b+'0');//wykonujemy jakie proste obliczenia i odsyamy ich wynik jako odpowied

				while(UCSR0B &(1<<UDRIE0));//czekamy na zakoczenie nadawania
				while(!(UCSR0A &(1<<TXC0)));//czekamy na zakoczenie nadawania

				DIR_ODB();//w trybie oczekiwania pozostajemy w trybie odbioru.
				_delay_ms(1);
			}
		}
	}
	
	return 0;
}

#endif
