/*
 * uart.c
 *
 *  Created on: 23-05-2013
 *      Author: Piotr Rzeszut
 *  
 * Description: Przykad obsugi UART z zastosowaniem przerwa i bufora koowego
 */
#include "uart.h"

volatile char UART_RX_BUF[UART_BUFFER_SIZE];
volatile uint8_t uart_rx_h=0;
volatile uint8_t uart_rx_t=0;

volatile char UART_TX_BUF[UART_BUFFER_SIZE];
volatile uint8_t uart_tx_h=0;
volatile uint8_t uart_tx_t=0;

volatile uint8_t rx_overrun=0;

void uart_init(uint16_t baud){
	//wyliczamy warto rejestru UBRR
	uint16_t ubrr_cal=F_CPU/16/baud-1;
	//konfiguracja baud
	UBRR0H = (uint8_t)(ubrr_cal>>8);
	UBRR0L = (uint8_t)ubrr_cal;
	//wczenie UART
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	UCSR0C = (1<<UCSZ00)|(1<<UCSZ01);
	UCSR0B |= (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);
}

void uart_putc(char data){
	uint8_t new_h=(uart_tx_h+1)&UART_BUFFER_MASK;
	while(new_h==uart_tx_t);
	UART_TX_BUF[new_h]=data;
	uart_tx_h=new_h;
	UCSR0B |= (1<<UDRIE0);
}

void uart_puts(char *string){
	char c;
	while((c=*string++))uart_putc(c);
}

void uart_putint(int number, uint8_t base){
	char buffer[20];
	itoa(number,buffer,base);
	uart_puts(buffer);
}

char uart_getc(void){
	if(uart_rx_t==uart_rx_h)return 0;
	uart_rx_t=(uart_rx_t+1)&UART_BUFFER_MASK;
	return UART_RX_BUF[uart_rx_t];
}

uint16_t uart_getint(void){
  char odebrany=0;
  uint16_t liczba=0;
  do{
  	odebrany=uart_getc();
    if(odebrany!=0){
      if(odebrany!='\r'){//filtrujemy znak zakoczenia liczby
	  	uart_putc(odebrany);
        liczba*=10;
        liczba+=odebrany-'0';
      }
    }
  }while(odebrany!='\r');//jeli odebralimy znak zakoczenia liczby to wychodzimy z ptli  
  return liczba;//zwracamy odebran liczb
}

ISR(USART0_RX_vect){
	uint8_t new_h=(uart_rx_h+1)&UART_BUFFER_MASK;

	if(new_h!=uart_rx_t){
		uart_rx_h=new_h;
		UART_RX_BUF[uart_rx_h]=UDR0;
	}else{
		rx_overrun=1;
		new_h=UDR0;//odczytujemy bufor, eby nie zawiesi systemu
	}
}

ISR(USART0_UDRE_vect){
	if(uart_tx_t!=uart_tx_h){
		uart_tx_t=(uart_tx_t+1)&UART_BUFFER_MASK;
		UDR0=UART_TX_BUF[uart_tx_t];
	}else{
		UCSR0B &= ~(1<<UDRIE0);
	}
}


